package Shipwright::Script::Help;
use strict;
use warnings;

use base 'App::CLI::Command::Help';

1;

__END__

=head1 NAME

Shipwright::Script::Help - Show help

=head1 SYNOPSIS

 help create

=head1 AUTHORS

sunnavy  C<< <sunnavy@bestpractical.com> >>

=head1 LICENCE AND COPYRIGHT

Shipwright is Copyright 2007-2012 Best Practical Solutions, LLC.

This program is free software; you can redistribute it and/or modify it
under the same terms as Perl itself.

